@@ -9,8 +9,11 @@
 module Replacer
   ( replaceProgram
   , replaceProgramFast
+  , replaceExpression
+  , replaceExpressionFast
   , ReplaceContext (..)
   , ReplaceProgramFunc
+  , ReplaceExpressionFunc
   )
 where
 
@@ -19,13 +22,15 @@ import Data.List (isPrefixOf)
 
 type ReplaceState a = (a, [Expression], [Expression -> Expression])
 
-type ReplaceExpressionFunc = ReplaceState Expression -> ReplaceContext -> ReplaceState Expression
+type ReplaceExpressionFunc' = ReplaceState Expression -> ReplaceContext -> ReplaceState Expression
 
 type ReplaceProgramFunc = ReplaceState Program -> Program
 
+type ReplaceExpressionFunc = ReplaceState Expression -> Expression
+
 newtype ReplaceContext = ReplaceCtx {_maxDepth :: Int}
 
-replaceBindings :: ReplaceState [Binding] -> ReplaceContext -> ReplaceExpressionFunc -> ReplaceState [Binding]
+replaceBindings :: ReplaceState [Binding] -> ReplaceContext -> ReplaceExpressionFunc' -> ReplaceState [Binding]
 replaceBindings state@(_, [], _) _ _ = state
 replaceBindings state@(_, _, []) _ _ = state
 replaceBindings state@([], _, _) _ _ = state
@@ -37,24 +42,24 @@ replaceBindings (bd : bds, ptns, repls) ctx func =
   let (bds', ptns', repls') = replaceBindings (bds, ptns, repls) ctx func
    in (bd : bds', ptns', repls')
 
-replaceExpression :: ReplaceExpressionFunc
-replaceExpression state@(expr, ptns@(ptn : _ptns), repls@(repl : _repls)) ctx =
+replaceExpression' :: ReplaceExpressionFunc'
+replaceExpression' state@(expr, ptns@(ptn : _ptns), repls@(repl : _repls)) ctx =
   if expr == ptn
-    then replaceExpression (repl expr, _ptns, _repls) ctx
+    then replaceExpression' (repl expr, _ptns, _repls) ctx
     else case expr of
       ExDispatch inner attr ->
-        let (expr', ptns', repls') = replaceExpression (inner, ptns, repls) ctx
+        let (expr', ptns', repls') = replaceExpression' (inner, ptns, repls) ctx
          in (ExDispatch expr' attr, ptns', repls')
       ExApplication inner tau ->
-        let (expr', ptns', repls') = replaceExpression (inner, ptns, repls) ctx
-         in case replaceBindings ([tau], ptns', repls') ctx replaceExpression of
+        let (expr', ptns', repls') = replaceExpression' (inner, ptns, repls) ctx
+         in case replaceBindings ([tau], ptns', repls') ctx replaceExpression' of
               ([tau'], ptns'', repls'') -> (ExApplication expr' tau', ptns'', repls'')
               (bds', _, _) -> error $ "Expected single binding, got " ++ show (length bds')
       ExFormation bds ->
-        let (bds', ptns', repls') = replaceBindings (bds, ptns, repls) ctx replaceExpression
+        let (bds', ptns', repls') = replaceBindings (bds, ptns, repls) ctx replaceExpression'
          in (ExFormation bds', ptns', repls')
       _ -> state
-replaceExpression state _ = state
+replaceExpression' state _ = state
 
 replaceBindingsFast :: [Binding] -> [Expression] -> [Expression] -> [Binding]
 replaceBindingsFast bds ((ExFormation pbds) : _ptns) ((ExFormation rbds) : _repls) =
@@ -69,35 +74,41 @@ replaceBindingsFast bds ((ExFormation pbds) : _ptns) ((ExFormation rbds) : _repl
       | otherwise = x : findAndReplace xs' ptn repl
 replaceBindingsFast bds _ _ = bds
 
-replaceExpressionFast :: ReplaceExpressionFunc
-replaceExpressionFast = replaceExpressionFast' 0
+replaceExpressionFast' :: ReplaceExpressionFunc'
+replaceExpressionFast' = _replaceExpressionFast 0
   where
-    replaceExpressionFast' :: Int -> ReplaceExpressionFunc
-    replaceExpressionFast' _ state@(_, [], _) _ = state
-    replaceExpressionFast' _ state@(_, _, []) _ = state
-    replaceExpressionFast' depth state@(expr, ptns, repls) ctx@ReplaceCtx{..} =
+    _replaceExpressionFast :: Int -> ReplaceExpressionFunc'
+    _replaceExpressionFast _ state@(_, [], _) _ = state
+    _replaceExpressionFast _ state@(_, _, []) _ = state
+    _replaceExpressionFast depth state@(expr, ptns, repls) ctx@ReplaceCtx{..} =
       if depth == _maxDepth
         then (expr, [], [])
         else case expr of
           ExFormation bds ->
             let replaced = replaceBindingsFast bds ptns (map (\rep -> rep expr) repls)
-                (bds', ptns', repls') = replaceBindings (replaced, ptns, repls) ctx (replaceExpressionFast' (depth + 1))
+                (bds', ptns', repls') = replaceBindings (replaced, ptns, repls) ctx (_replaceExpressionFast (depth + 1))
              in (ExFormation bds', ptns', repls')
           ExDispatch inner attr ->
-            let (expr', ptns', repls') = replaceExpressionFast (inner, ptns, repls) ctx
+            let (expr', ptns', repls') = replaceExpressionFast' (inner, ptns, repls) ctx
              in (ExDispatch expr' attr, ptns', repls')
           ExApplication inner (BiTau attr arg) ->
-            let (expr', ptns', repls') = replaceExpressionFast (inner, ptns, repls) ctx
-                (expr'', ptns'', repls'') = replaceExpressionFast (arg, ptns', repls') ctx
+            let (expr', ptns', repls') = replaceExpressionFast' (inner, ptns, repls) ctx
+                (expr'', ptns'', repls'') = replaceExpressionFast' (arg, ptns', repls') ctx
              in (ExApplication expr' (BiTau attr expr''), ptns'', repls'')
           _ -> state
 
+replaceExpression :: ReplaceExpressionFunc
+replaceExpression state =
+  let (expr, _, _) = replaceExpression' state (ReplaceCtx 0)
+   in expr
+
+replaceExpressionFast :: ReplaceContext -> ReplaceExpressionFunc
+replaceExpressionFast ctx state =
+  let (expr, _, _) = replaceExpressionFast' state ctx
+   in expr
+
 replaceProgram :: ReplaceProgramFunc
-replaceProgram (Program expr, ptns, repls) =
-  let (expr', _, _) = replaceExpression (expr, ptns, repls) (ReplaceCtx 0)
-   in Program expr'
+replaceProgram (Program expr, ptns, repls) = Program (replaceExpression (expr, ptns, repls))
 
 replaceProgramFast :: ReplaceContext -> ReplaceProgramFunc
-replaceProgramFast ctx (Program expr, ptns, repls) =
-  let (expr', _, _) = replaceExpressionFast (expr, ptns, repls) ctx
-   in Program expr'
+replaceProgramFast ctx (Program expr, ptns, repls) = Program (replaceExpressionFast ctx (expr, ptns, repls))
