@@ -21,20 +21,21 @@ import System.Directory (removeFile)
 import System.Exit (ExitCode (ExitSuccess))
 import System.FilePath (makeRelative)
 import System.IO (hClose, hPutStr, openTempFile)
+import System.Info (os)
 import System.Process (readProcessWithExitCode)
 import Test.Hspec (Spec, anyException, describe, expectationFailure, it, pendingWith, runIO, shouldBe, shouldThrow)
 import XMIR (defaultXmirContext, parseXMIRThrows, printXMIR, programToXMIR, xmirToPhi)
 
 data ParsePack = ParsePack
-  { failure :: Maybe Bool,
-    xmir :: String,
-    phi :: String
+  { failure :: Maybe Bool
+  , xmir :: String
+  , phi :: String
   }
   deriving (Generic, Show, FromJSON)
 
 data PrintPack = PrintPack
-  { phi :: String,
-    xpaths :: [String]
+  { phi :: String
+  , xpaths :: [String]
   }
   deriving (Generic, Show, FromJSON)
 
@@ -50,6 +51,10 @@ isXmllintAvailable =
   let (exitCode, _, _) = unsafePerformIO (readProcessWithExitCode "xmllint" ["--version"] "")
    in (exitCode == ExitSuccess)
 
+-- Check if running on Windows
+isWindows :: Bool
+isWindows = os == "mingw32"
+
 spec :: Spec
 spec = do
   describe "XMIR parsing packs" $ do
@@ -59,7 +64,7 @@ spec = do
       packs
       ( \pth -> it (makeRelative resources pth) $ do
           pack <- parsePack pth
-          let ParsePack {phi = phi'} = pack
+          let ParsePack{phi = phi'} = pack
               xmir' = do
                 doc <- parseXMIRThrows (xmir pack)
                 xmirToPhi doc
@@ -73,14 +78,14 @@ spec = do
 
   describe "prohibit to convert to XMIR" $
     forM_
-      [ "[[ ]]",
-        "T",
-        "[[ x -> ? ]]",
-        "[[ ^ -> 5 ]]",
-        "Q.x.y.z",
-        "\"Hello\"",
-        "Q",
-        "$"
+      [ "[[ ]]"
+      , "T"
+      , "[[ x -> ? ]]"
+      , "[[ ^ -> 5 ]]"
+      , "Q.x.y.z"
+      , "\"Hello\""
+      , "Q"
+      , "$"
       ]
       ( \phi' -> it phi' $ do
           expr <- parseExpressionThrows phi'
@@ -95,29 +100,32 @@ spec = do
       packs
       ( \pth ->
           it (makeRelative resources pth) $
-            if not available
-              then pendingWith "The 'xmllint' is not available"
-              else do
-                pack <- printPack pth
-                let PrintPack {phi = phi', xpaths = xpaths'} = pack
-                prog <- parseProgramThrows phi'
-                xmir' <- programToXMIR prog defaultXmirContext
-                let xml = printXMIR xmir'
-                bracket
-                  (openTempFile "." "xmirXXXXXX.tmp")
-                  (\(fp, _) -> removeFile fp)
-                  ( \(path, hTmp) -> do
-                      hPutStr hTmp xml
-                      hClose hTmp
-                      failed <-
-                        filterM
-                          ( \xpath -> do
-                              (code, _, _) <- readProcessWithExitCode "xmllint" ["--xpath", xpath, path] ""
-                              pure (code /= ExitSuccess)
-                          )
-                          xpaths'
-                      unless
-                        (null failed)
-                        (expectationFailure ("Failed xpaths:\n - " ++ intercalate "\n - " failed ++ "\nXMIR is:\n" ++ xml))
-                  )
+            if isWindows
+              then pendingWith "Skipped on Windows due to xmllint Unicode issues"
+              else
+                if not available
+                  then pendingWith "The 'xmllint' is not available"
+                  else do
+                    pack <- printPack pth
+                    let PrintPack{phi = phi', xpaths = xpaths'} = pack
+                    prog <- parseProgramThrows phi'
+                    xmir' <- programToXMIR prog defaultXmirContext
+                    let xml = printXMIR xmir'
+                    bracket
+                      (openTempFile "." "xmirXXXXXX.tmp")
+                      (\(fp, _) -> removeFile fp)
+                      ( \(path, hTmp) -> do
+                          hPutStr hTmp xml
+                          hClose hTmp
+                          failed <-
+                            filterM
+                              ( \xpath -> do
+                                  (code, _, _) <- readProcessWithExitCode "xmllint" ["--xpath", xpath, path] ""
+                                  pure (code /= ExitSuccess)
+                              )
+                              xpaths'
+                          unless
+                            (null failed)
+                            (expectationFailure ("Failed xpaths:\n - " ++ intercalate "\n - " failed ++ "\nXMIR is:\n" ++ xml))
+                      )
       )
